﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;

namespace HeuristicLab.Hive.Contracts {
  [DataContract]
  public class StreamedObject<T> : Stream {
    private T encapsulatedObject;

    private Stream stream = 
      new MemoryStream();

    public StreamedObject(T encapsulatedObject) {
      this.encapsulatedObject = encapsulatedObject;

      BinaryFormatter formatter = new BinaryFormatter();
      formatter.Serialize(this.stream, this.encapsulatedObject);

      this.stream.Seek(0, SeekOrigin.Begin);
    }

    public T EncapsulatedObject {
      get {
        return this.encapsulatedObject;
      }
    }

    public override bool CanRead {
      get { return this.stream.CanRead; }
    }

    public override bool CanSeek {
      get { throw new NotImplementedException(); }
    }

    public override bool CanWrite {
      get { throw new NotImplementedException(); }
    }

    public override void Flush() {
      throw new NotImplementedException();
    }

    public override long Length {
      get { return this.stream.Length; }
    }

    public override long Position {
      get {
        return this.stream.Position;
      }
      set {
        if(this.stream.Position != value)
          throw new NotImplementedException();
      }
    }

    public override int Read(byte[] buffer, int offset, int count) {
      int read = this.stream.Read(buffer, offset, count);

      return read;
    }

    public override long Seek(long offset, SeekOrigin origin) {
      throw new NotImplementedException();
    }

    public override void SetLength(long value) {
      throw new NotImplementedException();
    }

    public override void Write(byte[] buffer, int offset, int count) {
      throw new NotImplementedException();
    }

    public override void Close() {
      this.stream.Close();
      base.Close();
    }
    protected override void Dispose(bool disposing) {
      this.stream.Dispose();
      base.Dispose(disposing);
    }
  }
}
